'\nExports function list_versions() for unicode version level support.\n\nThis code generated by wcwidth/bin/update-tables.py on 2025-09-15 16:57:50 UTC.\n'
def list_versions():'\n\tReturn Unicode version levels supported by this module release.\n\n\tAny of the version strings returned may be used as keyword argument\n\t``unicode_version`` to the ``wcwidth()`` family of functions.\n\n\t:returns: Supported Unicode version numbers in ascending sorted order.\n\t:rtype: list[str]\n\t';return'4.1.0','5.0.0','5.1.0','5.2.0','6.0.0','6.1.0','6.2.0','6.3.0','7.0.0','8.0.0','9.0.0','10.0.0','11.0.0','12.0.0','12.1.0','13.0.0','14.0.0','15.0.0','15.1.0','16.0.0','17.0.0'